@@ -474,7 +474,7 @@ bool LogFunction::representsRational(const MathStructure&, bool) const {return f
 bool LogFunction::representsReal(const MathStructure &vargs, bool) const {return vargs.size() == 1 && vargs[0].representsReal() && vargs[0].representsPositive();}
 bool LogFunction::representsNonComplex(const MathStructure &vargs, bool) const {return vargs.size() == 1 && vargs[0].representsReal() && vargs[0].representsNonNegative();}
 bool LogFunction::representsComplex(const MathStructure &vargs, bool) const {return vargs.size() == 1 && vargs[0].representsReal() && vargs[0].representsNegative();}
-bool LogFunction::representsNonZero(const MathStructure &vargs, bool) const {return vargs.size() == 1 && (vargs[0].representsNonPositive() || (vargs[0].isNumber() && COMPARISON_IS_NOT_EQUAL(vargs[0].number().compare(nr_one))) || (vargs[0].isVariable() && vargs[0].variable()->isKnown() && ((KnownVariable*) vargs[0].variable())->get().isNumber() && COMPARISON_IS_NOT_EQUAL(((KnownVariable*) vargs[0].variable())->get().number().compare(nr_one))));}
+bool LogFunction::representsNonZero(const MathStructure &vargs, bool) const {return vargs.size() == 1 && (vargs[0].representsNonPositive() || (vargs[0].isNumber() && comparison_is_not_equal(vargs[0].number().compare(nr_one))) || (vargs[0].isVariable() && vargs[0].variable()->isKnown() && ((KnownVariable*) vargs[0].variable())->get().isNumber() && comparison_is_not_equal(((KnownVariable*) vargs[0].variable())->get().number().compare(nr_one))));}
 bool LogFunction::representsEven(const MathStructure&, bool) const {return false;}
 bool LogFunction::representsOdd(const MathStructure&, bool) const {return false;}
 bool LogFunction::representsUndefined(const MathStructure&) const {return false;}
@@ -704,7 +704,10 @@ int LognFunction::calculate(MathStructure &mstruct, const MathStructure &vargs,
 	if(mstruct.isVector()) return -1;
 	MathStructure mstructv2 = vargs[1];
 	mstructv2.eval(eo);
-	if(mstructv2.isVector()) return -2;
+	if(mstructv2.isVector()) {
+		mstruct.transform(STRUCT_VECTOR, mstructv2);
+		return -3;
+	}
 	if(mstruct.isPower()) {
 		if((mstruct[0].representsPositive(true) && mstruct[1].representsReal()) || (mstruct[1].isNumber() && mstruct[1].number().isFraction())) {
 			MathStructure mstruct2;
