@@ -281,25 +281,37 @@ int AbsFunction::calculate(MathStructure &mstruct, const MathStructure &vargs, c
 	}
 	return -1;
 }
-GcdFunction::GcdFunction() : MathFunction("gcd", 2) {
+GcdFunction::GcdFunction() : MathFunction("gcd", 2, -1) {
 	RATIONAL_POLYNOMIAL_ARGUMENT_HV(1)
 	RATIONAL_POLYNOMIAL_ARGUMENT_HV(2)
+	RATIONAL_POLYNOMIAL_ARGUMENT(3)
 }
 int GcdFunction::calculate(MathStructure &mstruct, const MathStructure &vargs, const EvaluationOptions &eo) {
-	if(MathStructure::gcd(vargs[0], vargs[1], mstruct, eo)) {
-		return 1;
+	mstruct = vargs[0];
+	MathStructure m1;
+	for(size_t i = 1; i < vargs.size(); i++) {
+		m1 = mstruct;
+		if(!MathStructure::gcd(m1, vargs[i], mstruct, eo)) {
+			return 0;
+		}
 	}
-	return 0;
+	return 1;
 }
-LcmFunction::LcmFunction() : MathFunction("lcm", 2) {
+LcmFunction::LcmFunction() : MathFunction("lcm", 2, -1) {
 	RATIONAL_POLYNOMIAL_ARGUMENT_HV(1)
 	RATIONAL_POLYNOMIAL_ARGUMENT_HV(2)
+	RATIONAL_POLYNOMIAL_ARGUMENT(3)
 }
 int LcmFunction::calculate(MathStructure &mstruct, const MathStructure &vargs, const EvaluationOptions &eo) {
-	if(MathStructure::lcm(vargs[0], vargs[1], mstruct, eo)) {
-		return 1;
+	mstruct = vargs[0];
+	MathStructure m1;
+	for(size_t i = 1; i < vargs.size(); i++) {
+		m1 = mstruct;
+		if(!MathStructure::lcm(m1, vargs[i], mstruct, eo)) {
+			return 0;
+		}
 	}
-	return 0;
+	return 1;
 }
 bool divisors_combine(MathStructure &m, vector<Number> factors, size_t skip, size_t pos = 0, Number nr = nr_one) {
 	for(; pos < factors.size() - skip; pos++) {
@@ -1262,6 +1274,41 @@ int BijectiveFunction::calculate(MathStructure &mstruct, const MathStructure &va
 	CALCULATOR->parse(&mstruct, vargs[0].symbol(), po);
 	return 1;
 }
+BinaryDecimalFunction::BinaryDecimalFunction() : MathFunction("bcd", 1, 2) {
+	setArgumentDefinition(1, new TextArgument());
+	setArgumentDefinition(2, new BooleanArgument());
+	setDefaultValue(2, "1");
+}
+int BinaryDecimalFunction::calculate(MathStructure &mstruct, const MathStructure &vargs, const EvaluationOptions &eo) {
+	bool b_packed = vargs[1].number().getBoolean();
+	if(vargs[0].symbol().find_first_of("23456789") != string::npos) {
+		CALCULATOR->parse(&mstruct, vargs[0].symbol(), eo.parse_options);
+		PrintOptions po; po.base = BASE_BINARY_DECIMAL; po.base_display = BASE_DISPLAY_NORMAL;
+		mstruct.eval(eo);
+		string str = mstruct.print(po);
+		if(!b_packed) {
+			size_t i = 0;
+			while(i < str.length()) {
+				str.insert(i, "0000");
+				i += 9;
+			}
+		}
+		mstruct.set(str, true, true);
+		return 1;
+	}
+	ParseOptions po = eo.parse_options;
+	po.base = BASE_BINARY_DECIMAL;
+	string str = vargs[0].symbol();
+	if(!b_packed) {
+		remove_blanks(str);
+		for(size_t i = 0; i < str.length(); i++) {
+			if(i % 8 > 3) str[str.length() - i - 1] = ' ';
+		}
+		remove_blanks(str);
+	}
+	CALCULATOR->parse(&mstruct, str, po);
+	return 1;
+}
 ImFunction::ImFunction() : MathFunction("im", 1) {
 	Argument *arg = new NumberArgument("", ARGUMENT_MIN_MAX_NONE, false, false);
 	arg->setHandleVector(true);
